//
// This is gaussj (Gauss-Jordan elimination) from NR 3/e,
// modified so as to avoid using nr3.h. It is used here to solve
// (a*+lambda*I)*db* = -g*, where a* is the scaled curvature
// matrix and -g* is the negative of the scaled gradient vector,
// here submitted as a 1-column matrix. On output, -g* is
// replaced by db*, the scaled correction vector (the solution to
// the equation), also as a 1-column matrix.
//
// November 01, 2014
//

#include "fit.h"

void gaussj(dmat& a, dmat& b)
{
  int i,icol,irow,j,k,l,ll;
  int n=a.size(), m=b[0].size();
  dbl big,dum,pivinv;

  ivec indxc(n),indxr(n),ipiv(n);
  for (j=0;j<n;j++) ipiv[j]=0;
  for (i=0;i<n;i++) {
    big=0.0;
    for (j=0;j<n;j++)
      if (ipiv[j] != 1)
        for (k=0;k<n;k++) {
          if (ipiv[k] == 0) {
            if (abs(a[j][k]) >= big) {
              big=abs(a[j][k]);
              irow=j;
              icol=k;
            }
          }
        }
    ++(ipiv[icol]);
    if (irow != icol) {
      for (l=0;l<n;l++) SWAP(a[irow][l],a[icol][l]);
      for (l=0;l<m;l++) SWAP(b[irow][l],b[icol][l]);
    }
    indxr[i]=irow;
    indxc[i]=icol;
    if (a[icol][icol] == 0.0) err("gaussj: Singular Matrix");
    pivinv=1.0/a[icol][icol];
    a[icol][icol]=1.0;
    for (l=0;l<n;l++) a[icol][l] *= pivinv;
    for (l=0;l<m;l++) b[icol][l] *= pivinv;
    for (ll=0;ll<n;ll++)
      if (ll != icol) {
        dum=a[ll][icol];
        a[ll][icol]=0.0;
        for (l=0;l<n;l++) a[ll][l] -= a[icol][l]*dum;
        for (l=0;l<m;l++) b[ll][l] -= b[icol][l]*dum;
      }
  }                                                     // end of main loop
  for (l=n-1;l>=0;l--) {
    if (indxr[l] != indxc[l])
      for (k=0;k<n;k++)
        SWAP(a[k][indxr[l]],a[k][indxc[l]]);
  }
}
