//
// scaling for g and a:
//
// With g and a input, scales both, using scale factors sf
// derived from the input elements of a, and returns the scaled
// elements of g and a in the arguments. Note it destroys
// unscaled g and a.
//
// Here, sf[j] = jth component of scaling factor = sqrt(a[j][j])
// scaled g[j] = g[j]/sf[j]
// scaled a[j][k] = a[j][k]/sqrt(sf[j]*sf[k])
//
#include "fit.h"

void scale(dvec& g, dmat& a, dvec& sf, int npar)
{
  int j,k;
  dbl prod;

  for (j=0;j<npar;j++) {                            // find sf[j] and g*[j]
    assert(a[j][j] > 0.0);
    sf[j] = sqrt(a[j][j]);
    g[j] /= sf[j];
  }

  for (j=0;j<npar;j++) {                                   // find a*[j][k]
    for (k=0;k<npar;k++) {
      prod = sf[j]*sf[k];
      assert(prod > 0.0);
      a[j][k] /= prod;
    }
  }
}

